#!/usr/bin/env python

"""
spritespreview.py

A thumbnail generator for KDE which interprets Acorn Spritefiles and
produces previews based on a single sprite from each file.

Copyright (C) 2003-2005 David Boddie <david@boddie.org.uk>

This software is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public
License along with this library; see the file COPYING
If not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""

# Only the QString and QImage classes are required from the Qt library
# and, even then, only implicitly.

#from qt import QString, QImage

# The ThumbCreator class is required since we will base our class on it.

import sip
from kio import ThumbCreator

# Import the spritefile module to handle the image conversions.

import spritefile

try:

    import Image
    Image.preinit()
    no_Image = 0

except ImportError:

    no_Image = 1

if no_Image == 0:

    try:
    
        from cStringIO import StringIO
    
    except ImportError:
    
        from StringIO import StringIO


# Define a class which will be initialised when the plugin is loaded.
# The name must be defined in the accompanying details.py file.

class SpritesPreviewClass(ThumbCreator):

    def create(self, path, width, height, image):
    
        # Create a preview of the file at "path" in a new QImage object
        # and assign it to the "image" reference passed.
        # The size of the image is arbitrary, although "width" and
        # "height" can be used if desired.
        
        # Use the spritefile module to create an image.
        
        try:
        
            # Open the file at the path specified.
            fp = open(str(path), "rb")
            
            # Read the Spritefile.
            s = spritefile.spritefile(fp)
            
            # Find the first sprite in the sprite area.
            name = s.sprites.keys()[0]
            
            # Retrieve the sprite details.
            sprite = s.sprites[name]
            
            if no_Image:
            
                # Obtain a PPM image which is equivalent to the sprite image,
                # but without any mask.
                data = self.convert(sprite)
            
            else:
            
                # Create an Image object, being careful not to rebind the "image"
                # variable.
                data = self.sprite_to_image(sprite)
            
            # Close the Spritefile.
            fp.close()
        
        except (IOError, KeyError, spritefile.spritefile_error):
        
            # If anything goes wrong with the loading or conversion then
            # indicate failure to the thumbnail IOSlave.
            return 0
        
        # Load the image data into the QImage object.
        image.loadFromData(data)
        
        # Return 1 on success; return 0 on failure.
        return 1
    
    def sprite_to_image(self, sprite):
    
        image = Image.fromstring(
            sprite['mode'], (sprite['width'], sprite['height']),
            sprite['image']
            )
        
        if sprite["dpi x"] != sprite["dpi y"]:
        
            # If the horizontal and vertical resolutions of the sprite are
            # not equal then rescale the image.
            if sprite["dpi x"] > sprite["dpi y"]:
            
                # There were fewer pixels per vertical unit of length so
                # increase the number of rows accordingly. This does not
                # try to find a new height which is an integer number of
                # pixels.
                yscale = float(sprite["dpi x"]) / sprite["dpi y"]
                image = image.resize( (image.size[0], image.size[1] * yscale) )
            
            else:
            
                # There were fewer pixels per vertical unit of length so
                # increase the number of rows accordingly. This does not
                # try to find a new height which is an integer number of
                # pixels.
                xscale = float(sprite["dpi x"]) / sprite["dpi y"]
                image = image.resize( (image.size[0] * xscale, image.size[1]) )
        
        # Put the data in a StringIO object.
        string_io = StringIO()
        
        for format in "PNG", "PPM", "TIFF", "BMP":
        
            if format in Image.SAVE.keys():
            
                string_io.seek(0)
                image.save(string_io, format = format)
                break
        
        if string_io.tell() == 0:
        
            # Use the convert method instead.
            return self.convert(sprite)
        
        string_io.seek(0)
        
        data = string_io.read()
        
        # Close the StringIO object.
        string_io.close()
        
        return data
    
    def convert(self, sprite):
    
        # Convert the sprite to a PPM image. Ideally, we would use the
        # Python Imaging Library to allow conversion to PNG files, but
        # the use of the relevant dynamically loaded libraries seems to
        # be difficult to configure correctly.
        
        width, height = sprite['width'], sprite['height']
        
        ppm_lines = []
        
        # Support rescaling for sprites with horizontal and vertical
        # resolutions that are not equal. The dots per inch in one direction
        # must be an integer multiple of the other for this to work.
        
        if sprite['dpi x'] > sprite['dpi y']:
        
            xfactor = 1
            yfactor = int(sprite['dpi x'] / sprite['dpi y'])
            ppm_lines.append(
                "P3 %i %i 255" % (width, yfactor * height)
                )
        
        elif sprite['dpi y'] > sprite['dpi x']:
        
            xfactor = int(sprite['dpi y'] / sprite['dpi x'])
            yfactor = 1
            ppm_lines.append(
                "P3 %i %i 255" % (xfactor * width, height)
                )
        
        else:
        
            xfactor = yfactor = 1
            ppm_lines.append("P3 %i %i 255" % (width, height))
        
        ptr = 0
        image = sprite['image']
        
        if sprite['mode'] == "RGB":
        
            for j in range(0, height):
            
                line = []
                
                for i in range(0, width):
                
                    line = line + [
                        "%i %i %i " % (
                            ord(image[ptr]), ord(image[ptr+1]), ord(image[ptr+2])
                            )
                        ] * xfactor
                    
                    ptr = ptr + 3
                
                ppm_lines = ppm_lines + ["".join(line)] * yfactor
        
        elif sprite['mode'] == "RGBA":
        
            for j in range(0, height):
            
                line = []
                
                for i in range(0, width):
                
                    if ord(image[ptr+3]) != 0:
                    
                        line = line + [
                            "%i %i %i " % (
                                ord(image[ptr]), ord(image[ptr+1]),
                                ord(image[ptr+2])
                                )
                            ] * xfactor
                    
                    else:
                    
                        line = line + ["0 0 0 "] * xfactor
                    
                    ptr = ptr + 4
                
                ppm_lines = ppm_lines + ["".join(line)] * yfactor
        
        else:
        
            #return QPixmap()
            raise KeyError
        
        ppm = "\n".join(ppm_lines)
        
        return ppm
